import { UnprocessableEntityException } from '@nestjs/common';
import { GetConfigurationArgs } from './dto/get-configuration.args';
import { SetConfigurationInput } from './dto/set-configuration.input';
import { Resolver, Mutation, Args, Query } from '@nestjs/graphql';
import { ConfigurationsService } from './configurations.service';
import { Configuration } from './entities/configuration.entity';
import { any, pipe, values } from 'ramda';
import { AccountRole, Roles } from '@nestjs-lib/auth';

@Roles(AccountRole.admin, AccountRole.super)
@Resolver(() => Configuration)
export class ConfigurationsResolver {
  constructor(private readonly configurationsService: ConfigurationsService) {}

  @Mutation(() => Configuration)
  setConfiguration(
    @Args('setConfigurationInput', { type: () => SetConfigurationInput })
    setConfigurationInput: SetConfigurationInput,
  ) {
    return this.configurationsService.setConfiguration(setConfigurationInput);
  }

  @Query(() => Configuration, { nullable: true })
  getConfiguration(
    @Args()
    getConfigurationArgs: GetConfigurationArgs,
  ) {
    if (
      pipe(
        values,
        any((value) => !value),
      )(getConfigurationArgs)
    ) {
      throw new UnprocessableEntityException('Must pass a parameter');
    }

    return this.configurationsService.findOneByConditions(getConfigurationArgs);
  }
}
