import { getRepositoryToken } from '@nestjs/typeorm';
import { Test, TestingModule } from '@nestjs/testing';
import { ConfigurationsService } from './configurations.service';
import { Configuration } from './entities/configuration.entity';
import { IsNull } from 'typeorm';
import { Etcd3 } from 'etcd3';

describe('ConfigurationsService', () => {
  let service: ConfigurationsService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        ConfigurationsService,
        {
          provide: getRepositoryToken(Configuration),
          useValue: {},
        },
        {
          provide: Etcd3,
          useValue: {},
        },
      ],
    }).compile();

    service = module.get<ConfigurationsService>(ConfigurationsService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('findOneByConditions', () => {
    it('should select by projectId only', async () => {
      const entity = new Configuration();
      const findOne = jest.fn<any, [any]>(() => Promise.resolve(entity));
      service['repository'].findOne = findOne;

      await expect(
        service.findOneByConditions({ projectId: 'uuid' }),
      ).resolves.toEqual(entity);

      expect(findOne.mock.calls[0][0]).toMatchObject({
        projectId: 'uuid',
        pipelineId: IsNull(),
      });
    });
  });
});
