import { GetConfigurationArgs } from './dto/get-configuration.args';
import { BaseDbService } from './../commons/services/base-db.service';
import { Injectable } from '@nestjs/common';
import { Configuration } from './entities/configuration.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { FindConditions, IsNull, Repository } from 'typeorm';
import { SetConfigurationInput } from './dto/set-configuration.input';
import { pick } from 'ramda';
import { Etcd3 } from 'etcd3';

@Injectable()
export class ConfigurationsService extends BaseDbService<Configuration> {
  constructor(
    @InjectRepository(Configuration)
    configurationRepository: Repository<Configuration>,
    private readonly etcd: Etcd3,
  ) {
    super(configurationRepository);
  }

  async setConfiguration(dto: SetConfigurationInput) {
    let entity = await this.repository.findOne(
      pick(['pipelineId', 'projectId'], dto),
    );
    if (!entity) {
      entity = this.repository.create(dto);
    }
    entity = await this.repository.save(entity);
    await this.syncToEtcd(entity);
    return entity;
  }

  async findOneByConditions(dto: FindConditions<GetConfigurationArgs>) {
    if (dto.projectId && !dto.pipelineId) {
      dto.pipelineId = IsNull();
    }
    return await this.repository.findOne(dto);
  }

  async syncToEtcd({ pipelineId, id }: { pipelineId?: string; id?: string }) {
    const config = await this.repository.findOneOrFail({
      where: { pipelineId, id },
      relations: ['pipeline', 'project'],
    });

    await this.etcd
      .put(`share/config/${config.id}`)
      .value(config.content)
      .exec();
    await this.etcd
      .put(`share/config/${config.pipeline.environment}/${config.project.name}`)
      .value(config.content)
      .exec();
  }
}
